@@ -2,6 +2,8 @@
 //
 // ProxyChannel.cxx
 //
+// $Id: ProxyChannel.cxx,v 1.39.2.127 2004/05/25 19:54:02 zvision Exp $
+//
 // Copyright (c) Citron Network Inc. 2001-2002
 //
 // This work is published under the GNU Public License (GPL)
@@ -20,26 +22,26 @@
 #pragma warning( disable : 4800 ) // warning about forcing value to bool
 #endif
 
-#include "gk.h"
-#include "gk_const.h"
+#include <ptlib.h>
+#include <q931.h>
+#include <h245.h>
+#include <h323pdu.h>
 #include "h323util.h"
-#include "Toolkit.h"
+#include "gk_const.h"
 #include "stl_supp.h"
+#include "Toolkit.h"
+#include "gk.h"
 #include "RasSrv.h"
 #include "GkClient.h"
-#include "ProxyChannel.h"
 #include "gkauth.h"
 #ifdef HAS_ACCT
 #include "gkacct.h"
 #endif
-#include <q931.h>
-#include <h245.h>
-#include <h323pdu.h>
+#include "ProxyChannel.h"
 
-const char *RoutedSec = "RoutedMode";
-const char *ProxySection = "Proxy";
-
-const char *H225_ProtocolID = "0.0.8.2250.0.2";
+const char* const RoutedSection = "RoutedMode";
+const char* const ProxySection = "Proxy";
+const char* const H225_ProtocolID = "0.0.8.2250.0.2";
 
 class PortRange {
 public:
@@ -66,7 +68,7 @@ void PortRange::LoadConfig(const char *sec, const char *setting, const char *def
 {
 	PStringArray cfgs = GkConfig()->GetString(sec, setting, def).Tokenise(",.:-/'", FALSE);
 	if (cfgs.GetSize() >= 2) // no such a setting in config
-		port = minport = cfgs[0].AsUnsigned(), maxport = cfgs[1].AsUnsigned();
+		port = minport = (WORD)cfgs[0].AsUnsigned(), maxport = (WORD)cfgs[1].AsUnsigned();
 	else
 		port = 0;
 	PTRACE_IF(2, port, setting << ": " << minport << '-' << maxport);
@@ -146,6 +148,10 @@ public:
 	// override from class ProxySocket
 	virtual Result ReceiveData();
 
+protected:
+	virtual bool WriteData(const BYTE* buffer, WORD len);
+	virtual bool FlushData(const BYTE* buffer, WORD len);
+		
 private:
 	// these should not be used
 	UDPProxySocket();
@@ -314,7 +320,7 @@ void CallSignalSocket::SetRemote(CallSignalSocket *socket)
 	remote = socket;
 	m_call = socket->m_call;
 	m_call->SetSocket(socket, this);
-	m_crv = (socket->m_crv & 0x7fffu);
+	m_crv = (WORD)(socket->m_crv & 0x7fffu);
 	m_h245Tunneling = socket->m_h245Tunneling;
 	socket->GetPeerAddress(peerAddr, peerPort);
 	localAddr = RasThread->GetLocalAddress(peerAddr);
@@ -426,18 +432,12 @@ ProxySocket::Result CallSignalSocket::ReceiveData()
 		PrintQ931(4, "Received:", q931pdu, psignal = &signal);
 
 		if( remote && body.GetTag() == H225_H323_UU_PDU_h323_message_body::e_setup ) {
-			const WORD newcrv = q931pdu->GetCallReference();
-			if( m_crv && (newcrv == (m_crv & 0x8000u)) ) {
+			const WORD newcrv = (WORD)q931pdu->GetCallReference();
+			if( m_crv && (newcrv == (m_crv & 0x7fffu)) ) {
 				PTRACE(3, "Q931\tWarning: duplicate Setup - ignored!");
 			} else {
 				PTRACE(4, "Q931\tMultiple calls over single signalling channel not supported - new connection needed");
 				
-				// indicate newConnectionNeeded
-				H225_CallTerminationCause cause;
-				cause.SetTag(H225_CallTerminationCause::e_releaseCompleteReason);
-				H225_ReleaseCompleteReason& releaseReason = cause;
-				releaseReason.SetTag(H225_ReleaseCompleteReason::e_newConnectionNeeded);
-				
 				Q931 releasePDU;
 				H225_H323_UserInformation userInfo;
 				H225_H323_UU_PDU_h323_message_body& msgBody = userInfo.m_h323_uu_pdu.m_h323_message_body;
@@ -445,8 +445,10 @@ ProxySocket::Result CallSignalSocket::ReceiveData()
 				H225_ReleaseComplete_UUIE& uuie = msgBody;
 				uuie.m_protocolIdentifier.SetValue(H225_ProtocolID);
 				uuie.IncludeOptionalField(H225_ReleaseComplete_UUIE::e_reason);
-				uuie.m_reason = cause;
-				releasePDU.BuildReleaseComplete(newcrv & 0x7fffu, TRUE);
+				uuie.m_reason.SetTag(H225_ReleaseCompleteReason::e_newConnectionNeeded);
+				if (((H225_Setup_UUIE&)body).HasOptionalField(H225_Setup_UUIE::e_callIdentifier))
+					uuie.m_callIdentifier = ((H225_Setup_UUIE&)body).m_callIdentifier;
+				releasePDU.BuildReleaseComplete(newcrv, TRUE);
 				SetUUIE(releasePDU, userInfo);
 				PrintQ931(5, "Send to " + Name(), &releasePDU, &userInfo);
 				
@@ -469,7 +471,7 @@ ProxySocket::Result CallSignalSocket::ReceiveData()
 		switch (body.GetTag())
 		{
 		case H225_H323_UU_PDU_h323_message_body::e_setup:
-			m_crv = (m_lastQ931->GetCallReference() | 0x8000u);
+			m_crv = (WORD)(m_lastQ931->GetCallReference() | 0x8000u);
 			m_setupUUIE = new H225_H323_UserInformation(signal);
 			changed = OnSetup(body);
 			break;
@@ -549,14 +551,14 @@ ProxySocket::Result CallSignalSocket::ReceiveData()
 	}
 
 	if (m_lastQ931->HasIE(Q931::DisplayIE)) {
-		PString display = GkConfig()->GetString(RoutedSec, "ScreenDisplayIE", "");
+		PString display = GkConfig()->GetString(RoutedSection, "ScreenDisplayIE", "");
 		if (!display) {
 			m_lastQ931->SetDisplayName(display);
 			changed = true;
 		}
 	}
 	if (m_lastQ931->HasIE(Q931::CallingPartyNumberIE)) {
-		PString newnumber = GkConfig()->GetString(RoutedSec, "ScreenCallingPartyNumberIE", "");
+		PString newnumber = GkConfig()->GetString(RoutedSection, "ScreenCallingPartyNumberIE", "");
 		if (!newnumber) {
 			unsigned plan, type;
 			PString oldnumber;
@@ -719,10 +721,13 @@ TCPProxySocket *CallSignalSocket::InternalConnectTo()
 		ForwardData();
 	} else {
 		PTRACE(3, "Q931\t" << peerAddr << ':' << peerPort << " DIDN'T ACCEPT THE CALL");
+		if( m_call )
+			m_call->SetSocket(this,NULL);
+		delete remote;
+		remote = NULL;
 		SendReleaseComplete(H225_ReleaseCompleteReason::e_unreachableDestination);
 		MarkBlocked(true);
 		CallTable::Instance()->RemoveCall(m_call);
-		delete remote;
 		MarkBlocked(false);
 	}
 	return remote;
@@ -769,17 +774,17 @@ TCPProxySocket *CallSignalSocket::ForwardCall()
 				Setup->SetCalledPartyNumber(AsString(a[n], FALSE));
 				break;
 			}
-		SetupUUIE.HasOptionalField(H225_Setup_UUIE::e_destinationAddress);
+		SetupUUIE.IncludeOptionalField(H225_Setup_UUIE::e_destinationAddress);
 		SetupUUIE.m_destinationAddress = a;
 	}
-	if (Toolkit::AsBool(GkConfig()->GetString(RoutedSec, "ShowForwarderNumber", "0")))
+	if (Toolkit::AsBool(GkConfig()->GetString(RoutedSection, "ShowForwarderNumber", "0")))
 		if (endptr forwarder = m_call->GetForwarder()) {
 			const H225_ArrayOf_AliasAddress & a = forwarder->GetAliases();
 			for (PINDEX n = 0; n < a.GetSize(); ++n)
 				if (a[n].GetTag() == H225_AliasAddress::e_dialedDigits) {
 					PString callingNumber(AsString(a[n], FALSE));
 					Setup->SetCallingPartyNumber(callingNumber);
-					SetupUUIE.HasOptionalField(H225_Setup_UUIE::e_sourceAddress);
+					SetupUUIE.IncludeOptionalField(H225_Setup_UUIE::e_sourceAddress);
 					SetupUUIE.m_sourceAddress.SetSize(1);
 					H323SetAliasAddress(callingNumber, SetupUUIE.m_sourceAddress[0]);
 					break;
@@ -888,26 +893,30 @@ bool CallSignalSocket::OnSetup(H225_Setup_UUIE & Setup)
 					m_call->SetCalledStationId(calledPartyNumber);
 			}
 			
-		if( m_call->GetCallingStationId().IsEmpty()
-			&& m_lastQ931->HasIE(Q931::CallingPartyNumberIE) ) {
+		if (m_call->GetCallingStationId().IsEmpty()
+			&& m_lastQ931->HasIE(Q931::CallingPartyNumberIE)) {
 			PString cli;
-			if( m_lastQ931->GetCallingPartyNumber(cli) ) {
-				endptr callingEP = m_call->GetCallingParty();
-				if( callingEP ) {
-					H225_ArrayOf_AliasAddress cliAliases;
-					cliAliases.SetSize(1);
-					H323SetAliasAddress(cli,cliAliases[0]);
-					if( callingEP->CompareAlias(&cliAliases) )
-						m_call->SetCallingStationId(cli);
-					else if( callingEP->IsGateway() 
-						&& dynamic_cast<GatewayRec*>(callingEP.operator->())->PrefixMatch(cliAliases) > 0 )
-						m_call->SetCallingStationId(cli);
-				} else if( Setup.HasOptionalField(H225_Setup_UUIE::e_sourceAddress) ) {
-					for( PINDEX i = 0; i < Setup.m_sourceAddress.GetSize(); i++ )
-						if( H323GetAliasAddressString(Setup.m_sourceAddress[i]) == cli ) {
+			if (m_lastQ931->GetCallingPartyNumber(cli)) {
+				if (Toolkit::AsBool(GkConfig()->GetString("Accounting", "AlwaysUseCLID", "0")))
+					m_call->SetCallingStationId(cli);
+				else {
+					endptr callingEP = m_call->GetCallingParty();
+					if (callingEP) {
+						H225_ArrayOf_AliasAddress cliAliases;
+						cliAliases.SetSize(1);
+						H323SetAliasAddress(cli,cliAliases[0]);
+						if (callingEP->CompareAlias(&cliAliases))
 							m_call->SetCallingStationId(cli);
-							break;
-						}
+						else if (callingEP->IsGateway() 
+							&& dynamic_cast<GatewayRec*>(callingEP.operator->())->PrefixMatch(cliAliases) > 0)
+							m_call->SetCallingStationId(cli);
+					} else if (Setup.HasOptionalField(H225_Setup_UUIE::e_sourceAddress)) {
+						for (PINDEX i = 0; i < Setup.m_sourceAddress.GetSize(); i++)
+							if (H323GetAliasAddressString(Setup.m_sourceAddress[i]) == cli) {
+								m_call->SetCallingStationId(cli);
+								break;
+							}
+					}
 				}
 			}
 		}
@@ -946,7 +955,8 @@ bool CallSignalSocket::OnSetup(H225_Setup_UUIE & Setup)
 		}
 		*/
 		{
-			ReadLock cfgLock(ConfigReloadMutex);
+//			already locked by proxy handler
+//			ReadLock cfgLock(ConfigReloadMutex);
 		
 			GkAuthenticatorList* authList = RasThread->GetAuthList();
 			if( authList )
@@ -993,9 +1003,27 @@ bool CallSignalSocket::OnSetup(H225_Setup_UUIE & Setup)
 
 		endptr called;
 
-		if (Setup.HasOptionalField(H225_Setup_UUIE::e_destCallSignalAddress))
-			called = RegistrationTable::Instance()->FindBySignalAdr(Setup.m_destCallSignalAddress);
-			
+		if (Setup.HasOptionalField(H225_Setup_UUIE::e_destCallSignalAddress)) {
+			PIPSocket::Address destAddr;
+			WORD destPort = 0;
+			if (GetIPAndPortFromTransportAddr(Setup.m_destCallSignalAddress, destAddr, destPort)
+				&& destAddr.IsValid() && destPort != 0) {
+				called = RegistrationTable::Instance()->FindBySignalAdr(Setup.m_destCallSignalAddress);
+				// if the destination signalling address is not the gatekeeper
+				// signalling address, add this address as an outer zone endpoint
+				if ((!called) && Toolkit::AsBool(GkConfig()->GetString("RasSrv::ARQFeatures", "CallUnregisteredEndpoints", "1"))) {
+					PIPSocket::Address _localAddr(0);
+					WORD _localPort = 0;
+					GetLocalAddress(_localAddr, _localPort);
+					if (_localPort != 0 && _localAddr.IsValid())
+						if (destPort != _localPort || destAddr != _localAddr)
+							called = RegistrationTable::Instance()->InsertRec(Setup);
+				}
+			} else
+				PTRACE(3, "Q931\tInvalid destination address found in the Q.931 Setup, ignored: "
+					<< Setup.m_destCallSignalAddress
+					);
+		}
 #ifdef HAS_ACCT
 		PString calledPartyNumber;
 		PString callingPartyNumber;
@@ -1038,6 +1066,43 @@ bool CallSignalSocket::OnSetup(H225_Setup_UUIE & Setup)
 #else
 			called = RegistrationTable::Instance()->FindEndpoint(Setup.m_destinationAddress, true);
 #endif
+			if (!called && Toolkit::AsBool(
+				GkConfig()->GetString("RasSrv::ARQFeatures", "ParseEmailAliases", "0"))) {
+				H225_TransportAddress destAddr;
+				PString destAlias;
+				PINDEX aliasIndex;
+				if (GetEmailAlias(Setup.m_destinationAddress, destAddr, 
+						&destAlias, &aliasIndex)) {
+					called = RegistrationTable::Instance()->FindBySignalAdr(destAddr);
+					if (called) {
+#ifdef HAS_ACCT
+						calledPartyNumber = H323GetAliasAddressString(
+							Setup.m_destinationAddress[aliasIndex]
+							);
+#endif
+						// remove the domain name part
+						H323SetAliasAddress(destAlias, 
+							Setup.m_destinationAddress[aliasIndex]
+							);
+					} else if (Toolkit::AsBool(
+						GkConfig()->GetString("RasSrv::ARQFeatures",
+							"CallUnregisteredEndpoints", "1"))) { 
+						// call to an unregistered endpoint
+						Setup.IncludeOptionalField(H225_Setup_UUIE::e_destCallSignalAddress);
+						Setup.m_destCallSignalAddress = destAddr;
+						called = RegistrationTable::Instance()->InsertRec(Setup);
+#ifdef HAS_ACCT
+						calledPartyNumber = H323GetAliasAddressString(
+							Setup.m_destinationAddress[aliasIndex]
+							);
+#endif
+						// remove the domain name part
+						H323SetAliasAddress(destAlias, 
+							Setup.m_destinationAddress[aliasIndex]
+							);
+					}
+				}
+			}
 		}
 
 		if (!called) {
@@ -1060,7 +1125,8 @@ bool CallSignalSocket::OnSetup(H225_Setup_UUIE & Setup)
 		long durationLimit = -1;
 		
 		{
-			ReadLock cfgLock(ConfigReloadMutex);
+//			already locked by proxy handler
+//			ReadLock cfgLock(ConfigReloadMutex);
 		
 			GkAuthenticatorList* authList = RasThread->GetAuthList();
 			if( authList )
@@ -1156,10 +1222,21 @@ bool CallSignalSocket::OnSetup(H225_Setup_UUIE & Setup)
 			Setup.IncludeOptionalField(H225_Setup_UUIE::e_sourceAddress);
 		Setup.m_sourceAddress.SetSize(s+1);
 		H323SetAliasAddress(H323ID, Setup.m_sourceAddress[s], H225_AliasAddress::e_h323_ID);
+	} else if (Setup.HasOptionalField(H225_Setup_UUIE::e_sourceAddress)) {
+		const PString display = GkConfig()->GetString(RoutedSection, "ScreenSourceAddress", "");
+		if (!display) {
+			Setup.m_sourceAddress.SetSize(1);
+			H323SetAliasAddress(display, Setup.m_sourceAddress[0]);
+		}
 	}
 
 	HandleH245Address(Setup);
 	HandleFastStart(Setup, true);
+	
+#if H225_PROTOCOL_VERSION >= 4
+	if (Setup.HasOptionalField(H225_Setup_UUIE::e_parallelH245Control) && m_h245handler)
+		OnTunneledH245(Setup.m_parallelH245Control);
+#endif
 	return true;
 }
 
@@ -1196,7 +1273,7 @@ bool CallSignalSocket::OnInformation(H225_Information_UUIE &)
 	return false; // do nothing
 }
 
-bool CallSignalSocket::OnReleaseComplete(H225_ReleaseComplete_UUIE & ReleaseComplete)
+bool CallSignalSocket::OnReleaseComplete(H225_ReleaseComplete_UUIE & /*ReleaseComplete*/)
 {
 	if( m_call ) {
 		m_call->SetDisconnectTime(time(NULL));
@@ -1218,7 +1295,7 @@ bool CallSignalSocket::OnFacility(H225_Facility_UUIE & Facility)
 					m_result = NoData;
 			break;
 		case H225_FacilityReason::e_callForwarded:
-			if (!Toolkit::AsBool(GkConfig()->GetString(RoutedSec, "ForwardOnFacility", "1")))
+			if (!Toolkit::AsBool(GkConfig()->GetString(RoutedSection, "ForwardOnFacility", "1")))
 				break;
 			fsocket = this;
 		case H225_FacilityReason::e_routeCallToGatekeeper:
@@ -1416,7 +1493,7 @@ bool CallSignalSocket::OnInformationMsg(Q931 & q931pdu)
 
 bool CallSignalSocket::SetH245Address(H225_TransportAddress & h245addr)
 {
-	if (m_h245Tunneling && Toolkit::AsBool(GkConfig()->GetString(RoutedSec, "RemoveH245AddressOnTunneling", "0")))
+	if (m_h245Tunneling && Toolkit::AsBool(GkConfig()->GetString(RoutedSection, "RemoveH245AddressOnTunneling", "0")))
 		return false;
 	if (!m_h245handler) // no H245 routed
 		return true;
@@ -1659,7 +1736,7 @@ bool H245Socket::ConnectRemote()
 
 H225_TransportAddress H245Socket::GetH245Address(const Address & myip)
 {
-	return SocketToH225TransportAddr(myip, listener ? listener->GetPort() : 0);
+	return SocketToH225TransportAddr(myip, (WORD)(listener ? listener->GetPort() : 0));
 }
 
 bool H245Socket::SetH245Address(H225_TransportAddress & h245addr, const Address & myip)
@@ -1795,7 +1872,7 @@ inline const H245_UnicastAddress_iPAddress & operator>>(const H245_UnicastAddres
 
 inline const H245_UnicastAddress_iPAddress & operator>>(const H245_UnicastAddress_iPAddress & addr, WORD & port)
 {
-	port = addr.m_tsapIdentifier;
+	port = (WORD)addr.m_tsapIdentifier;
 	return addr;
 }
 
@@ -1909,7 +1986,7 @@ ProxySocket::Result UDPProxySocket::ReceiveData()
 	Address fromIP;
 	WORD fromPort;
 	GetLastReceiveAddress(fromIP, fromPort);
-	buflen = GetLastReadCount();
+	buflen = (WORD)GetLastReadCount();
 
 	// Workaround: some bad endpoints don't send packets from the specified port
 	if ((fromIP == fSrcIP || fromIP == rDestIP) && (fromIP != rSrcIP || fromPort == fSrcPort)) {
@@ -1926,6 +2003,52 @@ ProxySocket::Result UDPProxySocket::ReceiveData()
 	return Forwarding;
 }
 
+bool UDPProxySocket::WriteData(const BYTE *buffer, WORD len)
+{
+	if (!IsSocketOpen())
+		return false;
+
+	if (GetQueueSize() > 0) {
+		QueuePacket(buffer, len);
+		PTRACE(3, Name() << " socket is busy, " << len << " bytes queued");
+		return false;
+	}
+	
+	if (Self()->Write(buffer, len)) {
+		PTRACE(6, Name() << ' ' << len << " bytes sent");
+		return true;
+	}
+
+	const WORD wcount = (WORD)Self()->GetLastWriteCount();
+	buffer += wcount;
+	len -= wcount;
+
+	PTRACE(4, Name() << " blocked, " << wcount << " bytes written, " << len << " bytes queued");
+	PutBackPacket(buffer, len);
+	return (wcount > 0) ? false : ErrorHandler(PSocket::LastWriteError);
+}
+
+bool UDPProxySocket::FlushData(const BYTE *buf, WORD len)
+{
+	if (!IsSocketOpen())
+		return false;
+
+	if (Self()->Write(buf, len)) {
+		PTRACE(6, Name() << ' ' << len << " bytes sent");
+		return true;
+	}
+
+	const WORD wcount = (WORD)Self()->GetLastWriteCount();
+	buf += wcount;
+	len -= wcount;
+
+	PutBackPacket(buf, len);
+
+	PTRACE(4, Name() << " blocked, " << wcount << " bytes written, " << len << " bytes queued");
+	return (wcount > 0) ? false : ErrorHandler(PSocket::LastWriteError);
+}
+
+
 // class T120ProxySocket
 T120ProxySocket::T120ProxySocket(T120ProxySocket *socket, WORD pt)
       : TCPProxySocket("T120", socket, pt)
@@ -1961,7 +2084,7 @@ RTPLogicalChannel::RTPLogicalChannel(WORD flcn, bool nated) : LogicalChannel(flc
 	const PIPSocket::Address addr = RasThread->GetGKHome();
 	// try to get an available port 10 times
 	for (int i = 0; i < 10; ++i) {
-		port = GetPortNumber();
+		port = (WORD)GetPortNumber();
 		// try to bind rtp to an even port and rtcp to the next one port
 		if ((addr == INADDR_ANY) ? (rtp->Bind(port) && rtcp->Bind(port + 1))
 			: (rtp->Bind(addr,port) && rtcp->Bind(addr,port + 1)))
@@ -2029,7 +2152,7 @@ void RTPLogicalChannel::HandleMediaChannel(H245_UnicastAddress_iPAddress *mediaC
 	}
 	UDPProxySocket::pMem SetDest = (reversed) ? &UDPProxySocket::SetReverseDestination : &UDPProxySocket::SetForwardDestination;
 	(rtcp->*SetDest)(tmpSrcIP, tmpSrcPort, *dest);
-	*mediaControlChannel << local << (port + 1);
+	*mediaControlChannel << local << (WORD)(port + 1);
 
 	if (mediaChannel) {
 		if (rev)
@@ -2221,12 +2344,12 @@ bool H245ProxyHandler::OnLogicalChannelParameters(H245_H2250LogicalChannelParame
 	if (!addr)
 		return false;
 	RTPLogicalChannel *lc = (flcn) ?
-		CreateRTPLogicalChannel(h225Params->m_sessionID, flcn) :
-		CreateFastStartLogicalChannel(h225Params->m_sessionID);
+		CreateRTPLogicalChannel((WORD)h225Params->m_sessionID, flcn) :
+		CreateFastStartLogicalChannel((WORD)h225Params->m_sessionID);
 	if (!lc)
 		return false;
 	lc->SetSource(*addr);
-	*addr << GetLocalAddr() << (lc->GetPort() + 1);
+	*addr << GetLocalAddr() << (WORD)(lc->GetPort() + 1);
 	if (h225Params->HasOptionalField(H245_H2250LogicalChannelParameters::e_mediaChannel)) {
 		H245_UnicastAddress_iPAddress *addr = GetH245UnicastAddress(h225Params->m_mediaChannel);
 		if (addr)
@@ -2239,7 +2362,7 @@ bool H245ProxyHandler::HandleOpenLogicalChannel(H245_OpenLogicalChannel & olc)
 {
 	if (hnat)
 		hnat->HandleOpenLogicalChannel(olc);
-	WORD flcn = olc.m_forwardLogicalChannelNumber;
+	const WORD flcn = (WORD)olc.m_forwardLogicalChannelNumber;
 	if (IsT120Channel(olc)) {
 		T120LogicalChannel *lc = CreateT120LogicalChannel(flcn);
 		if (olc.HasOptionalField(H245_OpenLogicalChannel::e_separateStack)
@@ -2257,7 +2380,7 @@ bool H245ProxyHandler::HandleOpenLogicalChannel(H245_OpenLogicalChannel & olc)
 
 bool H245ProxyHandler::HandleOpenLogicalChannelReject(H245_OpenLogicalChannelReject & olcr)
 {
-	peer->RemoveLogicalChannel(olcr.m_forwardLogicalChannelNumber);
+	peer->RemoveLogicalChannel((WORD)olcr.m_forwardLogicalChannelNumber);
 	return false; // nothing changed :)
 }
 
@@ -2265,7 +2388,7 @@ bool H245ProxyHandler::HandleOpenLogicalChannelAck(H245_OpenLogicalChannelAck &
 {
 	if (hnat)
 		hnat->HandleOpenLogicalChannelAck(olca);
-	WORD flcn = olca.m_forwardLogicalChannelNumber;
+	const WORD flcn = (WORD)olca.m_forwardLogicalChannelNumber;
 	LogicalChannel *lc = peer->FindLogicalChannel(flcn);
 	if (!lc) {
 		PTRACE(2, "Proxy\tWarning: logical channel " << flcn << " not found");
@@ -2286,7 +2409,8 @@ bool H245ProxyHandler::HandleCloseLogicalChannel(H245_CloseLogicalChannel & clc)
 		first = this, second = peer;
 	else
 		first = peer, second = this;
-	first->RemoveLogicalChannel(clc.m_forwardLogicalChannelNumber) || second->RemoveLogicalChannel(clc.m_forwardLogicalChannelNumber);
+	first->RemoveLogicalChannel((WORD)clc.m_forwardLogicalChannelNumber) 
+		|| second->RemoveLogicalChannel((WORD)clc.m_forwardLogicalChannelNumber);
 	return false; // nothing changed :)
 }
 
@@ -2307,12 +2431,12 @@ bool H245ProxyHandler::HandleFastStartResponse(H245_OpenLogicalChannel & olc)
 		return false;
 	if (hnat)
 		hnat->HandleOpenLogicalChannel(olc);
-	WORD flcn = olc.m_forwardLogicalChannelNumber;
+	const WORD flcn = (WORD)olc.m_forwardLogicalChannelNumber;
 	bool changed = false, isReverseLC;
 	H245_H2250LogicalChannelParameters *h225Params = GetLogicalChannelParameters(olc, isReverseLC);
 	if (!h225Params)
 		return false;
-	WORD id = h225Params->m_sessionID;
+	WORD id = (WORD)h225Params->m_sessionID;
 	siterator iter = peer->fastStartLCs.find(id);
 	RTPLogicalChannel *lc = (iter != peer->fastStartLCs.end()) ? iter->second : 0;
 	if (isReverseLC) {
@@ -2368,8 +2492,8 @@ RTPLogicalChannel *H245ProxyHandler::CreateRTPLogicalChannel(WORD id, WORD flcn)
 		PTRACE(3, "Proxy\tRTP logical channel " << flcn << " already exist?");
 		return 0;
 	}
-	RTPLogicalChannel *lc;
-	if ((lc = peer->FindRTPLogicalChannelBySessionID(id)) && !lc->IsAttached()) {
+	RTPLogicalChannel *lc = peer->FindRTPLogicalChannelBySessionID(id);
+	if (lc && !lc->IsAttached()) {
 		lc = new RTPLogicalChannel(lc, flcn, hnat != 0);
 	// if H.245 OpenLogicalChannel is received, the fast connect procedure
 	// should be disable. So we reuse the fast start logical channel here
@@ -2502,12 +2626,12 @@ TCPProxySocket *ProxyListener::CreateSocket()
 // class HandlerList
 void HandlerList::LoadConfig()
 {
-	Q931PortRange.LoadConfig(RoutedSec, "Q931PortRange");
-	H245PortRange.LoadConfig(RoutedSec, "H245PortRange");
+	Q931PortRange.LoadConfig(RoutedSection, "Q931PortRange");
+	H245PortRange.LoadConfig(RoutedSection, "H245PortRange");
 	T120PortRange.LoadConfig(ProxySection, "T120PortRange");
 	RTPPortRange.LoadConfig(ProxySection, "RTPPortRange", "10000-59999");
 
-	WORD port = GkConfig()->GetInteger(RoutedSec, "CallSignalPort", GK_DEF_CALL_SIGNAL_PORT);
+	WORD port = (WORD)GkConfig()->GetInteger(RoutedSection, "CallSignalPort", GK_DEF_CALL_SIGNAL_PORT);
 	if (!listenerThread || (GKPort != port)) {
 		CloseListener();
 		unsigned queueSize = GkConfig()->GetInteger("ListenQueueLength", GK_DEF_LISTEN_QUEUE_LENGTH);
@@ -2516,7 +2640,7 @@ void HandlerList::LoadConfig()
 	}
 
 	// the handler number can only be increased
-	int s = GkConfig()->GetInteger(RoutedSec, "CallSignalHandlerNumber", 1);
+	int s = GkConfig()->GetInteger(RoutedSection, "CallSignalHandlerNumber", 1);
 	if (s < 1) { // sanity check
 		PTRACE(3, "Proxy\tCallSignalHandlerNumber must be >= 1");
 		s = 1;
