@@ -2,6 +2,8 @@
 //
 // ProxyThread.cxx
 //
+// $Id: ProxyThread.cxx,v 1.15.2.45 2004/05/25 19:54:03 zvision Exp $
+//
 // Copyright (c) Citron Network Inc. 2001-2002
 //
 // This work is published under the GNU Public License (GPL)
@@ -20,11 +22,13 @@
 #pragma warning( disable : 4800 ) // warning about forcing value to bool
 #endif
 
-#include "ProxyThread.h"
+#include <ptlib.h>
+#include <ptlib/sockets.h>
 #include "h323util.h"
 #include "stl_supp.h"
 #include "rwlock.h"
 #include "gk.h"
+#include "ProxyThread.h"
 
 
 class ProxyConnectThread : public MyPThread {
@@ -76,6 +80,8 @@ ProxySocket::~ProxySocket()
 	PWaitAndSignal lock(writeMutex);
 	delete [] wbuffer;
 	DeleteObjectsInContainer<PBYTEArray>(queue);
+	queue.clear();
+	qsize = 0;
 	PTRACE(3, type << "\tDelete socket " << name);
 }
 
@@ -86,7 +92,7 @@ ProxySocket::Result ProxySocket::ReceiveData()
 		return NoData;
 	}
 	PTRACE(6, type << "\tReading from " << Name());
-	buflen = self->GetLastReadCount();
+	buflen = (WORD)self->GetLastReadCount();
 	return Forwarding;
 }
 
@@ -97,7 +103,7 @@ bool ProxySocket::ForwardData()
 
 bool ProxySocket::TransmitData(const PBYTEArray & buf)
 {
-	return WriteData(buf, buf.GetSize());
+	return WriteData(buf, (WORD)buf.GetSize());
 }
 
 bool ProxySocket::TransmitData(const BYTE *buf, WORD len)
@@ -115,45 +121,26 @@ bool ProxySocket::EndSession()
 bool ProxySocket::Flush()
 {
 	while (qsize > 0) {
-		queueMutex.Wait();
-		PBYTEArray *pdata = queue.front();
-		queue.pop_front();
-		--qsize;
-		queueMutex.Signal();
-		PBYTEArray data(*pdata);
-		delete pdata;
-		if (!WriteData(data, data.GetSize()))
+		PBYTEArray* const packet = PopQueuedPacket();
+		if (packet == NULL)
+			break;
+		PBYTEArray data(*packet);
+		delete packet;
+		if (!FlushData(data, (WORD)data.GetSize()))
 			return false;
-		PTRACE(4, type << '\t' << data.GetSize() << " be flushed to " << Name());
+		PTRACE(4, type << '\t' << data.GetSize() << " bytes flushed to " << Name());
 	}
 	return true;
 }
 
-void ProxySocket::AddToSelectList(SocketSelectList & sl)
-{
-	sl.Append(self);
-}
-
-bool ProxySocket::IsReadable(int milisec)
-{
-	return SocketSelectList(self).Select(SocketSelectList::Read, milisec);
-}
-
-bool ProxySocket::IsWriteable(int milisec)
-{
-	return SocketSelectList(self).Select(SocketSelectList::Write, milisec);
-}
-
 bool ProxySocket::WriteData(const BYTE *buf, WORD len)
 {
 	if (!IsSocketOpen())
 		return false;
 
-	if (writeMutex.WillBlock()) {
+	if (qsize > 0 || writeMutex.WillBlock()) {
+		QueuePacket(buf, len);
 		PTRACE(3, Name() << " socket is busy, " << len << " bytes queued");
-		PWaitAndSignal qlock(queueMutex);
-		queue.push_back(new PBYTEArray(buf, len));
-		++qsize;
 		return false;
 	}
 
@@ -164,17 +151,43 @@ bool ProxySocket::WriteData(const BYTE *buf, WORD len)
 		return true;
 	}
 
-	WORD wcount = self->GetLastWriteCount();
-	buf += wcount, len -= wcount;
+	const WORD wcount = (WORD)self->GetLastWriteCount();
+	buf += wcount;
+	len -= wcount;
 
+	PutBackPacket(buf, len);
 	PTRACE(4, Name() << " blocked, " << wcount << " bytes written, " << len << " bytes queued");
-	queueMutex.Wait();
-	queue.push_front(new PBYTEArray(buf, len));
-	++qsize;
-	queueMutex.Signal();
 	return (wcount > 0) ? false : ErrorHandler(PSocket::LastWriteError);
 }
 
+bool ProxySocket::FlushData(const BYTE *buf, WORD len)
+{
+	if (!IsSocketOpen())
+		return false;
+
+	if (!writeMutex.WillBlock()) {
+		PWaitAndSignal lock(writeMutex);
+
+		if (self->Write(buf, len)) {
+			PTRACE(6, Name() << ' ' << len << " bytes sent");
+			return true;
+		}
+
+		const WORD wcount = (WORD)self->GetLastWriteCount();
+		buf += wcount;
+		len -= wcount;
+
+		PutBackPacket(buf, len);
+
+		PTRACE(4, Name() << " blocked, " << wcount << " bytes written, " << len << " bytes queued");
+		return (wcount > 0) ? false : ErrorHandler(PSocket::LastWriteError);
+	} else {
+		PutBackPacket(buf, len);
+		PTRACE(3, Name() << " socket is busy, " << len << " bytes queued");
+		return false;
+	}
+}
+
 bool ProxySocket::ErrorHandler(PSocket::ErrorGroup group)
 {
 	PSocket::Errors e = self->GetErrorCode(group);
@@ -362,7 +375,7 @@ bool TCPProxySocket::ReadTPKT()
 			PTRACE(2, "Proxy\t" << Name() << " NOT TPKT PACKET!");
 			return false; // Only support version 3
 		}
-		buflen = PIPSocket::Net2Host(tpkt.length) - sizeof(TPKTV3);
+		buflen = (WORD)(PIPSocket::Net2Host(tpkt.length) - sizeof(TPKTV3));
 		if (buflen < 1) {
 			PTRACE(3, "Proxy\t" << Name() << " PACKET TOO SHORT!");
 			buflen = 0;
@@ -376,7 +389,7 @@ bool TCPProxySocket::ReadTPKT()
 #ifdef LARGE_FDSET
 	// some endpoints may send TPKT header and payload in separate
 	// packets, so we have to check again if data available
-	if (!YaSelectList(this).Select(YaSelectList::Read, 0))
+	if (!CanRead(0))
 		return false;
 #endif
 	if (!Read(bufptr, buflen))
@@ -392,7 +405,8 @@ bool TCPProxySocket::ReadTPKT()
 
 bool TCPProxySocket::InternalWrite(const PBYTEArray & buf)
 {
-	WORD len = buf.GetSize(), tlen = len + sizeof(TPKTV3);
+	const WORD len = (WORD)buf.GetSize();
+	const WORD tlen = (WORD)(len + sizeof(TPKTV3));
 	PBYTEArray tbuf(tlen);
 	BYTE *bptr = tbuf.GetPointer();
 	new (bptr) TPKTV3(len); // placement operator
@@ -558,7 +572,8 @@ void ProxyHandleThread::FlushSockets()
 	       return;
 	
 	PTRACE(5, "Proxy\t" << wlist.GetSize() << " sockets to flush...");
-	for (int k = 0; k < wlist.GetSize(); ++k) {
+	const int wsize = wlist.GetSize();
+	for (int k = 0; k < wsize; ++k) {
 		ProxySocket *socket = dynamic_cast<ProxySocket *>(wlist[k]);
 		if (socket->Flush()) {
 			PTRACE(4, "Proxy\t" << socket->Name() << " flush ok");
@@ -630,13 +645,14 @@ void ProxyHandleThread::BuildSelectList(SocketSelectList & result)
 
 void ProxyHandleThread::Exec()
 {
-	int ss;
+	int ss = 0;
 	ReadLock cfglock(ConfigReloadMutex);
 	SocketSelectList sList;
 	while (true) {
 		FlushSockets();
 		BuildSelectList(sList);
-		if ((ss = sList.GetSize())) {
+		ss = sList.GetSize();
+		if (ss > 0) {
 			ReadUnlock cfglock(ConfigReloadMutex);
 			if (sList.Select(SocketSelectList::Read, PTimeInterval(100)))
 				break;
