@@ -2,6 +2,8 @@
 //
 // bookkeeping for RAS-Server in H.323 gatekeeper
 //
+// $Id: RasTbl.cxx,v 1.50.2.94 2004/05/25 19:54:03 zvision Exp $
+//
 // This work is published under the GNU Public License (GPL)
 // see file COPYING for details.
 // We also explicitely grant the right to link this code
@@ -22,50 +24,76 @@
 #define snprintf	_snprintf
 #endif
 
-#include <time.h>
 #include <ptlib.h>
+#include <ptlib/sockets.h>
+#include <time.h>
+#include <h225.h>
 #include <h323pdu.h>
-#include "ANSI.h"
+#include "gk_const.h"
 #include "h323util.h"
 #include "Toolkit.h"
-#include "SoftPBX.h"
-#include "RasSrv.h"
-#include "GkClient.h"
 #include "stl_supp.h"
+#include "SoftPBX.h"
+#include "GkClient.h"
 #include "ProxyChannel.h"
-#include "gk_const.h"
+#include "RasSrv.h"
+#include "RasTbl.h"
 
-#define DEFAULT_CONNECT_TIMEOUT 180000
+const char* const CallTableSection = "CallTable";
 
-const char *CallTableSection = "CallTable";
-const char *RRQFeaturesSection = "RasSrv::RRQFeatures";
+namespace {
+const long DEFAULT_CONNECT_TIMEOUT = 180000;
+const char* const RRQFeaturesSection = "RasSrv::RRQFeatures";
+}
 
-
-EndpointRec::EndpointRec(const H225_RasMessage &completeRAS, bool Permanent)
-      :	m_RasMsg(completeRAS), m_timeToLive(1), m_pollCount(2),
-	m_nat(false), m_natsocket(0)
+EndpointRec::EndpointRec(
+	/// RRQ, ARQ, ACF or LCF that contains a description of the endpoint
+	const H225_RasMessage& ras, 
+	/// permanent endpoint flag
+	bool permanent
+	)
+	: m_RasMsg(ras), m_timeToLive(1),  
+	m_activeCall(0), m_connectedCall(0), m_totalCall(0), 
+	m_pollCount(2), m_usedCount(0),
+	m_nat(false), m_natsocket(0), m_permanent(permanent)
 {
-	m_activeCall = m_connectedCall = m_totalCall = m_usedCount = 0;
 	switch (m_RasMsg.GetTag())
 	{
-		case H225_RasMessage::e_registrationRequest:
-			SetEndpointRec((H225_RegistrationRequest &)m_RasMsg);
-			PTRACE(1, "New EP|" << PrintOn(false));
-			break;
-		case H225_RasMessage::e_admissionRequest:
-			SetEndpointRec((H225_AdmissionRequest &)m_RasMsg);
-			break;
-		case H225_RasMessage::e_admissionConfirm:
-			SetEndpointRec((H225_AdmissionConfirm &)m_RasMsg);
-			break;
-		case H225_RasMessage::e_locationConfirm:
-			SetEndpointRec((H225_LocationConfirm &)m_RasMsg);
-			break;
-		default: // should not happen
-			break;
+	case H225_RasMessage::e_registrationRequest:
+		SetEndpointRec((H225_RegistrationRequest &)m_RasMsg);
+		PTRACE(1, "New EP|" << PrintOn(false));
+		break;
+	case H225_RasMessage::e_admissionRequest:
+		SetEndpointRec((H225_AdmissionRequest &)m_RasMsg);
+		break;
+	case H225_RasMessage::e_admissionConfirm:
+		SetEndpointRec((H225_AdmissionConfirm &)m_RasMsg);
+		break;
+	case H225_RasMessage::e_locationConfirm:
+		SetEndpointRec((H225_LocationConfirm &)m_RasMsg);
+		break;
+	default: // should not happen
+		PTRACE(1, "RAS\tCannot build an endpoint from RAS message "<<m_RasMsg.GetTagName());
 	}
-	if (Permanent)
+	if (m_permanent)
 		m_timeToLive = 0;
+
+	LoadEndpointConfig();
+}	
+
+EndpointRec::EndpointRec(
+	/// Setup-UUIE with the destination signalling address
+	const H225_Setup_UUIE& setup,
+	/// unique identifier for this endpoint
+	const H225_EndpointIdentifier& epId
+	)
+	: m_timeToLive(1), m_activeCall(0), m_connectedCall(0), m_totalCall(0), 
+	m_pollCount(2), m_usedCount(0), m_nat(false), m_natsocket(0), m_permanent(false)
+{
+	// to avoid misuse of m_RasMsg
+	m_RasMsg.SetTag(H225_RasMessage::e_nonStandardMessage);
+	SetEndpointRec(setup, epId);
+	LoadEndpointConfig();
 }	
 
 void EndpointRec::SetEndpointRec(H225_RegistrationRequest & rrq)
@@ -127,6 +155,33 @@ void EndpointRec::SetEndpointRec(H225_LocationConfirm & lcf)
 	m_fromParent = false;
 }
 
+void EndpointRec::SetEndpointRec(
+	/// Setup-UUIE with the destination signalling address for the endpoint
+	const H225_Setup_UUIE& setup,
+	/// unique identifier for this endpoint
+	const H225_EndpointIdentifier& epId
+	)
+{
+	// as Setup-UUIE does not contain any information about
+	// destination endpoint type, use this dummy variable
+	static H225_EndpointType termType;
+	// we set it to non-standard address to avoid misuse
+	m_rasAddress.SetTag(H225_TransportAddress::e_nonStandardAddress);
+	// should always be present
+	if (setup.HasOptionalField(H225_Setup_UUIE::e_destCallSignalAddress))
+		m_callSignalAddress = setup.m_destCallSignalAddress;
+	else {
+		PTRACE(1, "RAS\tTrying to build an outer zone endpoint from Setup-UUIE"
+			" that does not contain destCallSignalAddress field!"
+			);
+		m_callSignalAddress.SetTag(H225_TransportAddress::e_nonStandardAddress);
+	}
+	m_endpointIdentifier = epId;
+	m_terminalType = &termType;
+	m_timeToLive = (SoftPBX::TimeToLive > 0) ? SoftPBX::TimeToLive : 600;
+	m_fromParent = false;
+}
+
 EndpointRec::~EndpointRec()
 {
 	PTRACE(3, "Gk\tDelete endpoint: " << m_endpointIdentifier.GetValue() << " " << m_usedCount);
@@ -134,6 +189,13 @@ EndpointRec::~EndpointRec()
 		m_natsocket->SetDeletable();
 }
 
+bool EndpointRec::LoadConfig()
+{ 
+	PWaitAndSignal lock(m_usedLock);
+	LoadEndpointConfig();
+	return true;
+}
+
 bool EndpointRec::PrefixMatch_IncompleteAddress(const H225_ArrayOf_AliasAddress &aliases, 
                                                bool &fullMatch) const
 {
@@ -159,12 +221,12 @@ bool EndpointRec::PrefixMatch_IncompleteAddress(const H225_ArrayOf_AliasAddress
 		  // check if it is a full match 
 		  if (aliasStr == reg_alias) {
 		    fullMatch = 1;
-  		    PTRACE(2, ANSI::DBG << "Alias " << aliasStr << " matches endpoint " 
-		      << (const unsigned char *)m_endpointIdentifier.GetValue() << " (full)" << ANSI::OFF);
+  		    PTRACE(2, "Alias " << aliasStr << " matches endpoint " 
+		      << (const unsigned char *)m_endpointIdentifier.GetValue() << " (full)");
   		  } else {
 		    partialMatch = 1;
-  		    PTRACE(2, ANSI::DBG << "Alias " << aliasStr << " matches endpoint " 
-		      << (const unsigned char *)m_endpointIdentifier.GetValue() << " (partial)" << ANSI::OFF);
+  		    PTRACE(2, "Alias " << aliasStr << " matches endpoint " 
+		      << (const unsigned char *)m_endpointIdentifier.GetValue() << " (partial)");
 		  }
 	        }
 //	      }
@@ -188,7 +250,7 @@ void EndpointRec::SetEndpointIdentifier(const H225_EndpointIdentifier &i)
         
 void EndpointRec::SetTimeToLive(int seconds)
 {
-	if (m_timeToLive > 0) {
+	if (m_timeToLive > 0 && !m_permanent) {
 		// To avoid bloated RRQ traffic, don't allow ttl < 60
 		if (seconds < 60)
 			seconds = 60;
@@ -198,27 +260,27 @@ void EndpointRec::SetTimeToLive(int seconds)
 	}
 }
 
-void EndpointRec::SetPermanent(bool b)
-{
-	PWaitAndSignal lock(m_usedLock);
-	m_timeToLive = (!b && SoftPBX::TimeToLive > 0) ? SoftPBX::TimeToLive : 0;
-}
-
 void EndpointRec::SetAliases(const H225_ArrayOf_AliasAddress &a)
 {
-	PWaitAndSignal lock(m_usedLock);
-        m_terminalAliases = a;
+	{
+		PWaitAndSignal lock(m_usedLock);
+		m_terminalAliases = a;
+	}
+	LoadConfig(); // update settings for the new aliases
 }
         
 void EndpointRec::SetEndpointType(const H225_EndpointType &t) 
 {
-	PWaitAndSignal lock(m_usedLock);
-        *m_terminalType = t;
+	{
+		PWaitAndSignal lock(m_usedLock);
+		*m_terminalType = t;
+	}
+	LoadConfig(); // update settings for the new endpoint type
 }
 
 void EndpointRec::Update(const H225_RasMessage & ras_msg)
 {
-        if (ras_msg.GetTag() == H225_RasMessage::e_registrationRequest) {
+	if (ras_msg.GetTag() == H225_RasMessage::e_registrationRequest) {
 		const H225_RegistrationRequest & rrq = ras_msg;
 
 		// don't update rasAddress for nated endpoint
@@ -286,7 +348,6 @@ void EndpointRec::BuildLCF(H225_LocationConfirm & obj_lcf) const
 {
 	obj_lcf.m_callSignalAddress = GetCallSignalAddress();
 	obj_lcf.m_rasAddress = GetRasAddress();
-	extern const char *LRQFeaturesSection;
 	if (Toolkit::AsBool(GkConfig()->GetString(LRQFeaturesSection, "IncludeDestinationInfoInLCF", "1"))) {
 		obj_lcf.IncludeOptionalField(H225_LocationConfirm::e_destinationInfo);
 		obj_lcf.m_destinationInfo = GetAliases();
@@ -304,7 +365,7 @@ PString EndpointRec::PrintOn(bool verbose) const
 		    (const unsigned char *) GetEndpointIdentifier().GetValue() );
 	if (verbose) {
 		msg += GetUpdatedTime().AsString();
-		if (m_timeToLive == 0)
+		if (m_permanent)
 			msg += " (permanent)";
 		PString natstring(m_nat ? m_natip.AsString() : PString());
 		msg += PString(PString::Printf, " C(%d/%d/%d) %s <%d>\r\n", m_activeCall, m_connectedCall, m_totalCall, (const unsigned char *)natstring, m_usedCount);
@@ -403,13 +464,7 @@ GatewayRec::GatewayRec(const H225_RasMessage &completeRRQ, bool Permanent)
       : EndpointRec(completeRRQ, Permanent), defaultGW(false)
 {
 	Prefixes.reserve(8);
-	GatewayRec::LoadConfig(); // static binding
-}
-
-void GatewayRec::SetAliases(const H225_ArrayOf_AliasAddress &a)
-{
-	EndpointRec::SetAliases(a);
-	LoadConfig();
+	LoadGatewayConfig();
 }
 
 void GatewayRec::SetEndpointType(const H225_EndpointType &t)
@@ -419,12 +474,11 @@ void GatewayRec::SetEndpointType(const H225_EndpointType &t)
 		return;
 	}
 	EndpointRec::SetEndpointType(t);
-	LoadConfig();
 }
 
 void GatewayRec::Update(const H225_RasMessage & ras_msg)
 {
-        if (ras_msg.GetTag() == H225_RasMessage::e_registrationRequest) {
+	if (ras_msg.GetTag() == H225_RasMessage::e_registrationRequest) {
 		const H225_RegistrationRequest & rrq = ras_msg;
 		if (!(rrq.HasOptionalField(H225_RegistrationRequest::e_keepAlive) && rrq.m_keepAlive))
 			SetEndpointType(rrq.m_terminalType);
@@ -474,24 +528,32 @@ void GatewayRec::AddPrefixes(const PString & prefixes)
 void GatewayRec::SortPrefixes()
 {
 	// remove duplicate aliases
-	sort(Prefixes.begin(), Prefixes.end(), greater<string>());
+	sort(Prefixes.begin(), Prefixes.end(), greater<std::string>());
 	prefix_iterator Iter = unique(Prefixes.begin(), Prefixes.end());
 	Prefixes.erase(Iter, Prefixes.end());
-	defaultGW = (find_if(Prefixes.begin(), Prefixes.end(), bind2nd(equal_to<string>(), "*")) != Prefixes.end());
-	defaultGW = (find(Prefixes.begin(), Prefixes.end(), string("*")) != Prefixes.end());
+	defaultGW = (find_if(Prefixes.begin(), Prefixes.end(), bind2nd(equal_to<std::string>(), "*")) != Prefixes.end());
+	defaultGW = (find(Prefixes.begin(), Prefixes.end(), std::string("*")) != Prefixes.end());
 }
 
 bool GatewayRec::LoadConfig()
 {
+	EndpointRec::LoadConfig();
 	PWaitAndSignal lock(m_usedLock);
+	LoadGatewayConfig();
+	return true;
+}
+
+void GatewayRec::LoadGatewayConfig()
+{
 	Prefixes.clear();
 	if (Toolkit::AsBool(GkConfig()->GetString(RRQFeaturesSection, "AcceptGatewayPrefixes", "1")))
 		if (m_terminalType->m_gateway.HasOptionalField(H225_GatewayInfo::e_protocol))
 			AddPrefixes(m_terminalType->m_gateway.m_protocol);
-	for (PINDEX i=0; i<m_terminalAliases.GetSize(); i++)
+	for (PINDEX i=0; i<m_terminalAliases.GetSize(); i++) {
 		AddPrefixes(GkConfig()->GetString("RasSrv::GWPrefixes", H323GetAliasAddressString(m_terminalAliases[i]), ""));
+		AddPrefixes(GkExtConfig()->GetString("RasSrv::GWPrefixes", H323GetAliasAddressString(m_terminalAliases[i]), ""));
+	}
 	SortPrefixes();
-	return true;
 }
 
 int GatewayRec::PrefixMatch(
@@ -595,7 +657,7 @@ PString GatewayRec::PrintOn(bool verbose) const
 		if (Prefixes.size() == 0) {
 			msg += "<none>";
 		} else {
-			string m=Prefixes.front();
+			std::string m=Prefixes.front();
 			const_prefix_iterator Iter = Prefixes.begin(), eIter= Prefixes.end();
 			while (++Iter != eIter)
 				m += "," + (*Iter);
@@ -612,6 +674,17 @@ OuterZoneEPRec::OuterZoneEPRec(const H225_RasMessage & completeRAS, const H225_E
 	PTRACE(1, "New OZEP|" << PrintOn(false));
 }
 
+OuterZoneEPRec::OuterZoneEPRec(
+	/// Setup-UUIE with the destination signalling address
+	const H225_Setup_UUIE& setup,
+	/// unique identifier for this endpoint
+	const H225_EndpointIdentifier& epId
+	) 
+	: EndpointRec(setup, epId)
+{
+	PTRACE(1,"New OZEP|" << PrintOn(false));
+}
+
 OuterZoneGWRec::OuterZoneGWRec(const H225_RasMessage & completeLCF, const H225_EndpointIdentifier &epID) : GatewayRec(completeLCF, false)
 {
 	m_endpointIdentifier = epID;
@@ -684,6 +757,24 @@ endptr RegistrationTable::InsertRec(H225_RasMessage & ras_msg, PIPSocket::Addres
 	return ep;
 }
 
+endptr RegistrationTable::InsertRec(
+	/// Setup-UUIE that contains destination signalling address of the endpoint
+	const H225_Setup_UUIE& setup
+	)
+{
+	endptr ep;
+	if (setup.HasOptionalField(H225_Setup_UUIE::e_destCallSignalAddress) 
+		&& !(ep = FindOZEPBySignalAdr(setup.m_destCallSignalAddress)))
+		ep = InternalInsertOZEP(setup);
+	else
+		PTRACE(2,"RAS\tUnable to insert outer zone endpoint ("
+			<< (setup.HasOptionalField(H225_Setup_UUIE::e_destCallSignalAddress)
+				?AsString(setup.m_destCallSignalAddress):PString("no address specified"))
+			<< ") into the registration table"
+			);
+	return ep;
+}
+
 endptr RegistrationTable::InternalInsertEP(H225_RasMessage & ras_msg)
 {
 	H225_RegistrationRequest & rrq = ras_msg;
@@ -702,8 +793,10 @@ endptr RegistrationTable::InternalInsertEP(H225_RasMessage & ras_msg)
 		GenerateAlias(rrq.m_terminalAlias, rrq.m_endpointIdentifier);
 	}
 
-	EndpointRec *ep = rrq.m_terminalType.HasOptionalField(H225_EndpointType::e_gateway) ?
-			  new GatewayRec(ras_msg) : new EndpointRec(ras_msg);
+	EndpointRec *ep = 
+		(rrq.m_terminalType.HasOptionalField(H225_EndpointType::e_gateway)
+			|| rrq.m_terminalType.HasOptionalField(H225_EndpointType::e_mcu)) 
+		? new GatewayRec(ras_msg) : new EndpointRec(ras_msg);
 	WriteLock lock(listLock);
 	EndpointList.push_back(ep);
 	++regSize;
@@ -727,7 +820,8 @@ endptr RegistrationTable::InternalInsertOZEP(H225_RasMessage & ras_msg, H225_Loc
 
 	EndpointRec *ep;
 	if (lcf.HasOptionalField(H225_LocationConfirm::e_destinationType) &&
-	    lcf.m_destinationType.HasOptionalField(H225_EndpointType::e_gateway))
+	    (lcf.m_destinationType.HasOptionalField(H225_EndpointType::e_gateway)
+			|| lcf.m_destinationType.HasOptionalField(H225_EndpointType::e_mcu)))
 		ep = new OuterZoneGWRec(ras_msg, epID);
 	else
 		ep = new OuterZoneEPRec(ras_msg, epID);
@@ -737,6 +831,19 @@ endptr RegistrationTable::InternalInsertOZEP(H225_RasMessage & ras_msg, H225_Loc
 	return endptr(ep);
 }
 
+endptr RegistrationTable::InternalInsertOZEP(
+	/// Setup-UUIE that contains destination signalling address of the endpoint
+	const H225_Setup_UUIE& setup
+	)
+{
+	H225_EndpointIdentifier epID;
+	epID = "oz_" + PString(PString::Unsigned, ozCnt++) + endpointIdSuffix;
+	EndpointRec *ep = new OuterZoneEPRec(setup, epID);
+	WriteLock lock(listLock);
+	OuterZoneList.push_front(ep);
+	return endptr(ep);
+}
+
 void RegistrationTable::RemoveByEndptr(const endptr & eptr)
 {
 	EndpointRec *ep = eptr.operator->(); // evil
@@ -841,7 +948,7 @@ endptr RegistrationTable::FindEndpoint(
 
 endptr RegistrationTable::InternalFindEP(
 	const H225_ArrayOf_AliasAddress& aliases,
-	list<EndpointRec *>* eplist, 
+	std::list<EndpointRec *>* eplist, 
 	bool roundrobin,
 	int* matchedalias
 	)
@@ -850,6 +957,7 @@ endptr RegistrationTable::InternalFindEP(
 
 	// if we also need to know matchedalias index, we need to do search "by hand"
 	if( matchedalias ) {
+		ReadLock lock(listLock);
 		const_iterator Iter = eplist->begin(), IterLast = eplist->end();
 		while (Iter != IterLast) {
 			if( (*Iter)->MatchAlias(aliases,*matchedalias) ) {
@@ -869,7 +977,7 @@ endptr RegistrationTable::InternalFindEP(
 
 	// build a list of gateways with the longest prefix match for the aliases
 	int maxlen = 0;
-	list<EndpointRec *> gwlist;
+	std::list<EndpointRec *> gwlist;
 	listLock.StartRead();
 	const_iterator Iter = eplist->begin(), IterLast = eplist->end();
 	while (Iter != IterLast) {
@@ -937,7 +1045,7 @@ void RegistrationTable::PrintRemoved(GkStatus::Client &client, BOOL verbose)
 	InternalPrint(client, verbose, &RemovedList, msg);
 }
 
-void RegistrationTable::InternalPrint(GkStatus::Client &client, BOOL verbose, list<EndpointRec *> * List, PString & msg)
+void RegistrationTable::InternalPrint(GkStatus::Client &client, BOOL verbose, std::list<EndpointRec *> * List, PString & msg)
 {
 	// copy the pointers into a temporary array to avoid large lock
 	listLock.StartRead();
@@ -959,7 +1067,7 @@ void RegistrationTable::InternalPrint(GkStatus::Client &client, BOOL verbose, li
 	client.WriteString(msg);
 }
 
-void RegistrationTable::InternalStatistics(const list<EndpointRec *> *List, unsigned & s, unsigned & t, unsigned & g, unsigned & n) const
+void RegistrationTable::InternalStatistics(const std::list<EndpointRec *> *List, unsigned & s, unsigned & t, unsigned & g, unsigned & n) const
 {
 	ReadLock lock(listLock);
 	s = List->size(), t = g = n = 0;
@@ -996,7 +1104,42 @@ void RegistrationTable::LoadConfig()
 	}
 
 	// Load permanent endpoints
-	PStringToString cfgs=GkConfig()->GetAllKeyValues("RasSrv::PermanentEndpoints");
+	PStringToString cfgs = GkConfig()->GetAllKeyValues("RasSrv::PermanentEndpoints");
+	{
+		PStringToString extCfg = GkExtConfig()->GetAllKeyValues("RasSrv::PermanentEndpoints");
+		for (PINDEX i = 0; i < extCfg.GetSize(); i++)
+			cfgs.SetAt(extCfg.GetKeyAt(i), extCfg.GetDataAt(i));
+	}
+	// first, remove permanent endpoints deleted from the config
+	{
+		WriteLock lock(listLock);
+		iterator iter = EndpointList.begin(), endIter = EndpointList.end();
+		while (iter != endIter) {
+			iterator epIter = iter++;
+			EndpointRec *ep = *epIter;
+			if (!ep->IsPermanent())
+				continue;
+			// find a corresponing permanent endpoint entry in the config file
+			const H225_TransportAddress& epSigAddr = ep->GetCallSignalAddress();
+			PINDEX i;
+			for (i = 0; i < cfgs.GetSize(); ++i) {
+				H225_TransportAddress taddr;
+				if (GetTransportAddress(cfgs.GetKeyAt(i), (WORD)GkConfig()->GetInteger("EndpointSignalPort", GK_DEF_ENDPOINT_SIGNAL_PORT), taddr))
+					if (taddr == epSigAddr)
+						break;
+			}
+			// if the entry has not been found, unregister this permanent endpoint
+			if (i >= cfgs.GetSize()) {
+				SoftPBX::DisconnectEndpoint(endptr(ep));
+				ep->Unregister();
+				RemovedList.push_back(ep);
+				EndpointList.erase(epIter);
+				--regSize;
+				PTRACE(2, "Permanent endpoint " << ep->GetEndpointIdentifier().GetValue() << " removed");
+			}
+		}
+	}
+	
 	for (PINDEX i = 0; i < cfgs.GetSize(); ++i) {
 		EndpointRec *ep;
 		H225_RasMessage rrq_ras;
@@ -1004,7 +1147,7 @@ void RegistrationTable::LoadConfig()
 		H225_RegistrationRequest &rrq = rrq_ras;
 
 		rrq.m_callSignalAddress.SetSize(1);
-		GetTransportAddress(cfgs.GetKeyAt(i), GkConfig()->GetInteger("EndpointSignalPort", GK_DEF_ENDPOINT_SIGNAL_PORT), rrq.m_callSignalAddress[0]);
+		GetTransportAddress(cfgs.GetKeyAt(i), (WORD)GkConfig()->GetInteger("EndpointSignalPort", GK_DEF_ENDPOINT_SIGNAL_PORT), rrq.m_callSignalAddress[0]);
 		endptr eptr = FindBySignalAdr(rrq.m_callSignalAddress[0]);
 
 		// a permanent endpoint may not support RAS
@@ -1137,14 +1280,16 @@ CallRec::CallRec(
 	m_srcInfo(AsString(arq.m_srcInfo)), 
 	m_bandWidth(bandwidth), m_CallNumber(0),
 	m_Called(calledep),
-	m_crv(arq.m_callReferenceValue.GetValue() & 0x7fffu),
+	m_crv((WORD)(arq.m_callReferenceValue.GetValue() & 0x7fffu)),
+	m_crvFromCalled((WORD)(arq.m_callReferenceValue.GetValue() & 0x7fffu)),
 	m_callingSocket(0), m_calledSocket(0),
 	m_usedCount(0), m_nattype(none), m_h245Routed(routeH245),
 	m_registered(false), m_forwarded(false),
 	m_setupTime(0), m_disconnectTime(0),
 	m_disconnectCause(0), m_srcSignalAddr(NULL), m_destSignalAddr(NULL)
 #ifdef HAS_ACCT
-	, m_acctEventsLogged(0)
+	, m_acctSessionId(Toolkit::Instance()->GenerateAcctSessionId()),
+	m_acctEventsLogged(0)
 #endif
 {
 	if( arq.HasOptionalField(H225_AdmissionRequest::e_destinationInfo) )
@@ -1237,7 +1382,8 @@ CallRec::CallRec(
 	m_conferenceIdentifier(setup.m_conferenceID),
 	m_bandWidth(bandwidth), m_CallNumber(0),
 	m_Called(calledEP),
-	m_crv(q931pdu.GetCallReference() & 0x7fffu),
+	m_crv((WORD)(q931pdu.GetCallReference() & 0x7fffu)),
+	m_crvFromCalled((WORD)(q931pdu.GetCallReference() & 0x7fffu)),
 	m_callingSocket(callingSocket), m_calledSocket(0),
 	m_usedCount(0), m_nattype(none), m_h245Routed(routeH245),
 	m_registered(false), m_forwarded(false),
@@ -1245,7 +1391,8 @@ CallRec::CallRec(
 	m_setupTime(setupTime), m_connectTime(0), m_disconnectTime(0),
 	m_disconnectCause(0), m_srcSignalAddr(NULL), m_destSignalAddr(NULL)
 #ifdef HAS_ACCT
-	, m_callingStationId((const char*)callingStationId),
+	, m_acctSessionId(Toolkit::Instance()->GenerateAcctSessionId()),
+	m_callingStationId((const char*)callingStationId),
 	m_calledStationId((const char*)calledStationId),
 	m_acctEventsLogged(0)
 #endif
@@ -1308,13 +1455,7 @@ CallRec::CallRec(
 		PIPSocket::Address addr;
 		WORD port = 0;
 		m_callingSocket->GetPeerAddress(addr,port);
-		
-		H225_TransportAddress_ipAddress& ipport = *m_srcSignalAddr;
-		ipport.m_ip[0] = addr[0];
-		ipport.m_ip[1] = addr[1];
-		ipport.m_ip[2] = addr[2];
-		ipport.m_ip[3] = addr[3];
-		ipport.m_port = port;
+		*m_srcSignalAddr = SocketToH225TransportAddr(addr, port);
 	}
 	
 	if( m_destSignalAddr == NULL && useDestCallSignalAddress )
@@ -1481,7 +1622,7 @@ int CallRec::CountEndpoints() const
 
 void CallRec::Disconnect(bool force)
 {
-	if ((force || Toolkit::AsBool(GkConfig()->GetString(RoutedSec, "DropCallsByReleaseComplete", "0"))) && (m_callingSocket || m_calledSocket))
+	if ((force || Toolkit::AsBool(GkConfig()->GetString(RoutedSection, "DropCallsByReleaseComplete", "0"))) && (m_callingSocket || m_calledSocket))
 		SendReleaseComplete();
 	else
 		SendDRQ();
@@ -1514,7 +1655,7 @@ void CallRec::BuildDRQ(H225_RasMessage & ras_msg, unsigned reason)
 	drq.m_conferenceID = m_conferenceIdentifier;
 	drq.IncludeOptionalField(H225_DisengageRequest::e_callIdentifier);
 	drq.m_callIdentifier = m_callIdentifier;
-	drq.m_callReferenceValue = m_Calling ? m_crv : (m_crv | 0x8000u);
+	drq.m_callReferenceValue = m_crv;
 }
 
 void CallRec::SendDRQ()
@@ -1528,6 +1669,7 @@ void CallRec::SendDRQ()
 // Warning: For an outer zone endpoint, the endpoint identifier may not correct
 	if (m_Called) {
 		drq.m_endpointIdentifier = m_Called->GetEndpointIdentifier();
+		drq.m_callReferenceValue = m_crvFromCalled;
 		RasThread->SendRas(ras_msg, m_Called->GetRasAddress());
 	}
 	if (m_Calling) {
@@ -1604,7 +1746,7 @@ PString CallRec::PrintOn(bool verbose) const
 		m_crv,
 		(const char*)m_destInfo,(const char*)m_srcInfo,
 		(const char *)GetEPString(m_Called, m_destSignalAddr), 
-		m_crv | 0x8000u,
+		m_crvFromCalled,
 		(const char*)m_destInfo,
 		(const char*)m_srcInfo
 	);
@@ -1722,12 +1864,7 @@ bool CallRec::SetSrcSignalAddr(
 		
 		delete m_srcSignalAddr;
 		m_srcSignalAddr = new H225_TransportAddress(H225_TransportAddress::e_ipAddress);
-		H225_TransportAddress_ipAddress& ipport = *m_srcSignalAddr;
-		ipport.m_ip[0] = addr[0];
-		ipport.m_ip[1] = addr[1];
-		ipport.m_ip[2] = addr[2];
-		ipport.m_ip[3] = addr[3];
-		ipport.m_port = port;
+		*m_srcSignalAddr = SocketToH225TransportAddr(addr, port);
 		return true;
 	} else
 		return false;
@@ -1784,12 +1921,7 @@ bool CallRec::SetDestSignalAddr(
 		
 		delete m_destSignalAddr;
 		m_destSignalAddr = new H225_TransportAddress(H225_TransportAddress::e_ipAddress);
-		H225_TransportAddress_ipAddress& ipport = *m_destSignalAddr;
-		ipport.m_ip[0] = addr[0];
-		ipport.m_ip[1] = addr[1];
-		ipport.m_ip[2] = addr[2];
-		ipport.m_ip[3] = addr[3];
-		ipport.m_port = port;
+		*m_destSignalAddr = SocketToH225TransportAddr(addr, port);
 		return true;
 	} else
 		return false;
@@ -1820,18 +1952,6 @@ void CallRec::SetCalledStationId( const PString& id )
 	m_calledStationId = id;
 }
 
-PString CallRec::GetAcctSessionId()
-{
-	PWaitAndSignal lock(m_usedLock);
-	return m_acctSessionId;
-}
-
-void CallRec::SetAcctSessionId( const PString& id )
-{
-	PWaitAndSignal lock(m_usedLock);
-	m_acctSessionId = id;
-}
-
 #endif
 
 CallTable::CallTable() : m_CallNumber(0), m_capacity(-1)
@@ -1858,7 +1978,7 @@ void CallTable::LoadConfig()
 	// because it is constant for all CallRecs and should not add another
 	// 4 bytes to each CallRec unnecessary
 	m_connectTimeout = PMAX(
-		GkConfig()->GetInteger(RoutedSec, "ConnectTimeout",DEFAULT_CONNECT_TIMEOUT),
+		GkConfig()->GetInteger(RoutedSection, "ConnectTimeout",DEFAULT_CONNECT_TIMEOUT),
 		5000
 		);
 	m_defaultDurationLimit = GkConfig()->GetInteger(
@@ -1922,9 +2042,14 @@ callptr CallTable::FindCallRec(const H225_CallIdentifier & CallId) const
 	return InternalFind(bind2nd(mem_fun(&CallRec::CompareCallId), &CallId));
 }
 
-callptr CallTable::FindCallRec(const H225_CallReferenceValue & CallRef) const
+callptr CallTable::FindCallRec(
+	const H225_CallReferenceValue& crv,
+	bool checkCrvFromCalled
+	) const
 {
-	return InternalFind(bind2nd(mem_fun(&CallRec::CompareCRV), CallRef.GetValue()));
+	return checkCrvFromCalled 
+		? InternalFind(bind2nd(mem_fun(&CallRec::CompareBothCRV), crv.GetValue()))
+		: InternalFind(bind2nd(mem_fun(&CallRec::CompareCRV), crv.GetValue()));
 }
 
 callptr CallTable::FindCallRec(PINDEX CallNumber) const
@@ -1955,13 +2080,15 @@ void CallTable::ClearTable()
 
 void CallTable::CheckCalls()
 {
+	// no need to lock ConfigReloadMutex here as it is already done 
+	// by the housekeeping thread
+
+	WriteLock lock(listLock);
 #ifdef HAS_ACCT
-	long acctinterval = m_acctUpdateInterval;
-	if( acctinterval > 0 )
-		ConfigReloadMutex.StartRead();
+	const long acctinterval = m_acctUpdateInterval;
 #endif
 	const time_t now = time(NULL);
-	WriteLock lock(listLock);
+	
 	iterator Iter = CallList.begin(), eIter = CallList.end();
 	while (Iter != eIter) {
 		iterator i = Iter++;
@@ -1970,24 +2097,19 @@ void CallTable::CheckCalls()
 			InternalRemove(i);
 		} 
 #ifdef HAS_ACCT
-		else if( acctinterval > 0 && (*i)->IsConnected() ) {
-			if( (now - (*i)->GetLastAcctUpdateTime()) >= acctinterval ) {
-				GkAcctLoggers* acct = Toolkit::Instance()->GetAcctList();
-				if( acct != NULL ) {
-					callptr cptr(*i);
-					if( !acct->LogAcctEvent( GkAcctLogger::AcctUpdate, cptr, now ) )
-						PTRACE(3,"GKACCT\tAcctUpdate event log failed for call no. "
-							<<(*i)->GetCallNumber()
-							);
-				}
+		else if (acctinterval > 0 && (*i)->IsConnected()
+			&& (now - (*i)->GetLastAcctUpdateTime()) >= acctinterval) {
+			GkAcctLoggers* const acct = Toolkit::Instance()->GetAcctList();
+			if (acct) {
+				callptr cptr(*i);
+				if (!acct->LogAcctEvent(GkAcctLogger::AcctUpdate, cptr, now))
+					PTRACE(3, "GKACCT\tAcctUpdate event log failed for call "
+						"no. " << (*i)->GetCallNumber()
+						);
 			}
 		}
 #endif
 	}
-#ifdef HAS_ACCT
-	if( acctinterval > 0 )
-		ConfigReloadMutex.EndRead();
-#endif
 
 	Iter = partition(RemovedList.begin(), RemovedList.end(), mem_fun(&CallRec::IsUsed));
 	for_each(Iter, RemovedList.end(), deleteobj<CallRec>());
@@ -1996,19 +2118,22 @@ void CallTable::CheckCalls()
 
 void CallTable::RemoveCall(const H225_DisengageRequest & obj_drq, const endptr & ep)
 {
-	callptr call = obj_drq.HasOptionalField(H225_DisengageRequest::e_callIdentifier) ? FindCallRec(obj_drq.m_callIdentifier) : FindCallRec(obj_drq.m_callReferenceValue.GetValue());
+	callptr call = obj_drq.HasOptionalField(H225_DisengageRequest::e_callIdentifier) 
+		? FindCallRec(obj_drq.m_callIdentifier) 
+		: FindCallRec(obj_drq.m_callReferenceValue, true);
 	if (call) {
 		if (ep == call->GetForwarder())
 			return;
 
 		if (ep != call->GetCallingParty() && ep != call->GetCalledParty()) {
-			PTRACE(3, "GK\tWarning: CallRec doesn't belong to the requesting endpoint!");
+			PTRACE(3, "GK\tWarning: CallRec doesn't belong to the endpoint that sent DRQ!");
 			return;
 		}
-		if (Toolkit::AsBool(GkConfig()->GetString(RoutedSec, "SendReleaseCompleteOnDRQ", "0")))
+		if (Toolkit::AsBool(GkConfig()->GetString(RoutedSection, "SendReleaseCompleteOnDRQ", "0")))
 			call->SendReleaseComplete(obj_drq.HasOptionalField(H225_DisengageRequest::e_terminationCause) ? &obj_drq.m_terminationCause : 0);
 		RemoveCall(call);
-	}
+	} else
+		PTRACE(3, "GK\tNo CallRec found for DRQ");
 }
 
 void CallTable::RemoveCall(const callptr & call)
@@ -2019,7 +2144,7 @@ void CallTable::RemoveCall(const callptr & call)
 
 bool CallTable::InternalRemovePtr(CallRec *call)
 {
-	PTRACE(6, ANSI::PIN << "GK\tRemoving callptr:" << AsString(call->GetCallIdentifier().m_guid) << "...\n" << ANSI::OFF);
+	PTRACE(6, "GK\tRemoving callptr:" << AsString(call->GetCallIdentifier().m_guid) << "...\n");
 	WriteLock lock(listLock);
 	InternalRemove(find(CallList.begin(), CallList.end(), call));
 	return true; // useless, workaround for VC
@@ -2027,7 +2152,7 @@ bool CallTable::InternalRemovePtr(CallRec *call)
 
 void CallTable::InternalRemove(const H225_CallIdentifier & CallId)
 {
-	PTRACE(5, ANSI::PIN << "GK\tRemoving CallId:" << AsString(CallId.m_guid) << "...\n" << ANSI::OFF);
+	PTRACE(5, "GK\tRemoving CallId:" << AsString(CallId.m_guid) << "...\n");
 	WriteLock lock(listLock);
 	InternalRemove(
 		find_if(CallList.begin(), CallList.end(),
@@ -2037,7 +2162,7 @@ void CallTable::InternalRemove(const H225_CallIdentifier & CallId)
 
 void CallTable::InternalRemove(unsigned CallRef)
 {
-	PTRACE(5, ANSI::PIN << "GK\tRemoving CallRef:" << CallRef << "...\n" << ANSI::OFF);
+	PTRACE(5, "GK\tRemoving CallRef:" << CallRef << "...\n");
 	WriteLock lock(listLock);
 	InternalRemove(
 		find_if(CallList.begin(), CallList.end(),
