@@ -13,6 +13,19 @@
  * with the OpenH323 library.
  *
  * $Log: radauth.h,v $
+ * Revision 1.1.2.19  2004/05/12 17:46:40  zvision
+ * Fixes CVS keyword substitution. New ParseEmailAliases config option.
+ *
+ * Revision 1.1.2.18  2004/05/12 14:00:47  zvision
+ * Header file usage more consistent. Solaris std::map problems fixed.
+ * Compilation warnings removed. VSNET2003 project files added. ANSI.h removed.
+ *
+ * Revision 1.1.2.17  2004/03/31 11:13:36  zvision
+ * New CheckSetupUnregisteredOnly option for RadAliasAuth module
+ *
+ * Revision 1.1.2.16  2004/02/18 20:45:16  zvision
+ * Prototype changed for GkAuthenticator::Check ARQ method
+ *
  * Revision 1.1.2.15  2004/01/07 20:37:26  zvision
  * RRQ endpoint alias control (add/remove) through RadAuth/RadAliasAuth modules
  *
@@ -62,11 +75,10 @@
  * Initial revision
  *
  */
-#ifndef __RADAUTH_H
-#define __RADAUTH_H
+#ifndef RADAUTH_H
+#define RADAUTH_H "#(@) $Id: radauth.h,v 1.1.2.19 2004/05/12 17:46:40 zvision Exp $"
 #ifdef HAS_RADIUS
 
-#include <ptlib.h>
 #include "gkauth.h"
 #include "radproto.h"
 
@@ -144,7 +156,7 @@ protected:
 	*/
 	virtual BOOL OnReceivedPDU(
 		RadiusPDU& pdu, /// received PDU
-		const H225_AdmissionRequest& arq, /// ARQ being processed
+		H225_AdmissionRequest& arq, /// ARQ being processed
 		unsigned& rejectReason /// reject reason on return FALSE
 		);
 		
@@ -157,7 +169,7 @@ protected:
 	*/
 	virtual BOOL OnReceivedPDU(
 		RadiusPDU& pdu, /// PDU received
-		const H225_AdmissionRequest& arq, /// ARQ being processed
+		H225_AdmissionRequest& arq, /// ARQ being processed
 		unsigned& rejectReason, /// reject reason on return FALSE
 		long& durationLimit /// call duration limit to set (-1 means no limit)
 		);
@@ -197,7 +209,7 @@ private:
 	*/
 	virtual int Check(
 		/// ARQ nessage to be authenticated
-		const H225_AdmissionRequest &, 
+		H225_AdmissionRequest &, 
 		/// reference to the variable, that can be set 
 		/// to custom H225_AdmissionRejectReason
 		/// if the check fails
@@ -349,7 +361,7 @@ protected:
 	*/
 	virtual BOOL OnReceivedPDU(
 		RadiusPDU& pdu, /// PDU received
-		const H225_AdmissionRequest& arq, /// ARQ being processed
+		H225_AdmissionRequest& arq, /// ARQ being processed
 		unsigned& rejectReason /// reject reason on return FALSE
 		);
 		
@@ -362,7 +374,7 @@ protected:
 	*/
 	virtual BOOL OnReceivedPDU(
 		RadiusPDU& pdu, /// PDU received
-		const H225_AdmissionRequest& arq, /// ARQ being processed
+		H225_AdmissionRequest& arq, /// ARQ being processed
 		unsigned& rejectReason, /// reject reason on return FALSE
 		long& durationLimit /// call duration limit to set (-1 means no limit)
 		);
@@ -376,8 +388,8 @@ protected:
 	*/
 	virtual BOOL OnReceivedPDU(
 		RadiusPDU& pdu, /// PDU received
-		const Q931& q931pdu, /// Q.931 Setup message being processed
-		const H225_Setup_UUIE& setup, /// H.225 Setup message being processed
+		Q931& q931pdu, /// Q.931 Setup message being processed
+		H225_Setup_UUIE& setup, /// H.225 Setup message being processed
 		unsigned& releaseCompleteCause, /// Q.931 cause on return FALSE
 		long& durationLimit /// call duration limit to set (-1 means no limit)
 		);
@@ -417,7 +429,7 @@ private:
 	*/
 	virtual int Check(
 		/// ARQ message to be authenticated
-		const H225_AdmissionRequest &, 
+		H225_AdmissionRequest &, 
 		/// reference to the variable, that can be set 
 		/// to custom H225_AdmissionRejectReason
 		/// if the check fails
@@ -435,11 +447,11 @@ private:
 	*/
 	virtual int Check(
 		/// received Q.931 Setup message
-		const Q931& q931pdu, 
+		Q931& q931pdu, 
 		/// received H.225 Setup message
-		const H225_Setup_UUIE& setup, 
+		H225_Setup_UUIE& setup, 
 		/// CallRec for the call being authenticated
-		const callptr& call,
+		callptr& call,
 		/// Q.931 cause to set, if authentication failed
 		unsigned& releaseCompleteCause, 
 		/// call duration limit to set (-1 for no duration limit)
@@ -470,7 +482,7 @@ private:
 	/// timeout (ms) for unused sockets to be deleted
 	unsigned socketDeleteTimeout;
 	/// how many times to transmit a single request (1==no retransmission)
-	/// to a single RADIUS server
+	/// 2to a single RADIUS server
 	unsigned numRequestRetransmissions;
 	/// retransmission fashion: 
 	/// 	FALSE - do #numRequestRetransmissions# for server A,
@@ -485,6 +497,9 @@ private:
 	/// if TRUE an h323-ivr-out attribute will be sent with every alias
 	/// found inside RRQ.m_terminalAlias
 	BOOL includeTerminalAliases;
+	/// check Q.931 Setup messages coming from unregistered endpoints only
+	/// and accept unconditionally Q.931 Setup messages from registered ones
+	BOOL checkSetupUnregisteredOnly;
 	/// local interface RADIUS client should be bound to (multihomed hosts)
 	PString localInterface;	
 	/// IP address of the local interface
@@ -500,4 +515,4 @@ private:
 };
 
 #endif /* HAS_RADIUS */
-#endif /* __RADAUTH_H */
+#endif /* RADAUTH_H */
