@@ -11,6 +11,26 @@
  * with the OpenH323 library.
  *
  * $Log: radproto.cxx,v $
+ * Revision 1.1.2.23  2004/05/25 19:54:05  zvision
+ * Some minor warnings removed
+ *
+ * Revision 1.1.2.22  2004/05/12 14:00:47  zvision
+ * Header file usage more consistent. Solaris std::map problems fixed.
+ * Compilation warnings removed. VSNET2003 project files added. ANSI.h removed.
+ *
+ * Revision 1.1.2.21  2004/04/21 14:18:13  zvision
+ * Default ports are now set to fixed values (1812, 1813), because of problems
+ * with getservbyname on some machines
+ *
+ * Revision 1.1.2.20  2004/04/20 15:47:52  zvision
+ * Descendancy check removed to allow compilation with RTTI enabled PWLib
+ *
+ * Revision 1.1.2.19  2004/04/15 13:01:34  zvision
+ * Fixed warnings for 64-bit machines and applied reinterpret_cast
+ *
+ * Revision 1.1.2.18  2004/04/12 22:29:39  zvision
+ * Removed loop unrolling
+ *
  * Revision 1.1.2.17  2003/12/21 12:20:38  zvision
  * Fixed conditional compilation
  *
@@ -79,7 +99,6 @@
 #include <ptlib/sockets.h>
 #include <ptclib/random.h>
 #include <ptclib/cypher.h>
-
 #include "radproto.h"
 
 #if PTRACING
@@ -642,11 +661,6 @@ PObject::Comparison RadiusAttr::Compare(
 	const PObject & obj   // Object to compare against.
 	) const
 {
-	if( !obj.IsDescendant(RadiusAttr::Class()) ) {
-		PAssertAlways(PInvalidCast);
-		return GreaterThan;
-	}
-
 	if( !(IsValid() && ((const RadiusAttr&)obj).IsValid()) ) {
 #ifdef _DEBUG
 		PAssertAlways(PInvalidParameter);
@@ -1973,7 +1987,7 @@ BOOL RadiusClient::SendRequest(
 
 BOOL RadiusClient::VerifyResponseAuthenticator(
 	const BYTE* requestBuffer,
-	PINDEX requestLength,
+	PINDEX /*requestLength*/,
 	const BYTE* responseBuffer,
 	PINDEX responseLength
 	)
@@ -2022,8 +2036,8 @@ BOOL RadiusClient::VerifyResponseAuthenticator(
 }
 
 BOOL RadiusClient::OnSendPDU( 
-	RadiusPDU& pdu,
-	BOOL retransmission,
+	RadiusPDU& /*pdu*/,
+	BOOL /*retransmission*/,
 	BOOL& changed
 	)
 {
@@ -2031,7 +2045,9 @@ BOOL RadiusClient::OnSendPDU(
 	return TRUE;
 }
 
-BOOL RadiusClient::OnReceivedPDU( RadiusPDU& pdu )
+BOOL RadiusClient::OnReceivedPDU( 
+	RadiusPDU& /*pdu*/
+	)
 {
 	return TRUE;
 }
@@ -2057,18 +2073,6 @@ RadiusClient::RAGenerator RadiusClient::GetRAGenerator(
 	return RAGeneratorRandom;
 }
 
-WORD RadiusClient::GetDefaultAuthPort()
-{
-	const WORD port = PSocket::GetPortByService( "udp", "radius" );
-	return (port==0) ? DefaultAuthPort : port;
-}
-
-WORD RadiusClient::GetDefaultAcctPort()
-{
-	const WORD port = PSocket::GetPortByService( "udp", "radacct" );
-	return (port==0) ? DefaultAcctPort : port;
-}
-
 void RadiusClient::FillRequestAuthenticator( 
 	RadiusPDU& pdu, 
 	const PString& secret,
@@ -2159,23 +2163,10 @@ void RadiusClient::EncryptPasswords(
 		const DWORD* buf2ptr = (const DWORD*)&digest;
 #endif
 
-		if( ((unsigned)buf2ptr & 3) || ((unsigned)buf1ptr & 3) ) {
-			((BYTE*)buf1ptr)[0] = ((BYTE*)buf1ptr)[0] ^ ((const BYTE*)buf2ptr)[0];
-			((BYTE*)buf1ptr)[1] = ((BYTE*)buf1ptr)[1] ^ ((const BYTE*)buf2ptr)[1];
-			((BYTE*)buf1ptr)[2] = ((BYTE*)buf1ptr)[2] ^ ((const BYTE*)buf2ptr)[2];
-			((BYTE*)buf1ptr)[3] = ((BYTE*)buf1ptr)[3] ^ ((const BYTE*)buf2ptr)[3];
-			((BYTE*)buf1ptr)[4] = ((BYTE*)buf1ptr)[4] ^ ((const BYTE*)buf2ptr)[4];
-			((BYTE*)buf1ptr)[5] = ((BYTE*)buf1ptr)[5] ^ ((const BYTE*)buf2ptr)[5];
-			((BYTE*)buf1ptr)[6] = ((BYTE*)buf1ptr)[6] ^ ((const BYTE*)buf2ptr)[6];
-			((BYTE*)buf1ptr)[7] = ((BYTE*)buf1ptr)[7] ^ ((const BYTE*)buf2ptr)[7];
-			((BYTE*)buf1ptr)[8] = ((BYTE*)buf1ptr)[8] ^ ((const BYTE*)buf2ptr)[8];
-			((BYTE*)buf1ptr)[9] = ((BYTE*)buf1ptr)[9] ^ ((const BYTE*)buf2ptr)[9];
-			((BYTE*)buf1ptr)[10] = ((BYTE*)buf1ptr)[10] ^ ((const BYTE*)buf2ptr)[10];
-			((BYTE*)buf1ptr)[11] = ((BYTE*)buf1ptr)[11] ^ ((const BYTE*)buf2ptr)[11];
-			((BYTE*)buf1ptr)[12] = ((BYTE*)buf1ptr)[12] ^ ((const BYTE*)buf2ptr)[12];
-			((BYTE*)buf1ptr)[13] = ((BYTE*)buf1ptr)[13] ^ ((const BYTE*)buf2ptr)[13];
-			((BYTE*)buf1ptr)[14] = ((BYTE*)buf1ptr)[14] ^ ((const BYTE*)buf2ptr)[14];
-			((BYTE*)buf1ptr)[15] = ((BYTE*)buf1ptr)[15] ^ ((const BYTE*)buf2ptr)[15];
+		if( (reinterpret_cast<unsigned long>(buf2ptr) & 3) 
+			|| (reinterpret_cast<unsigned long>(buf1ptr) & 3) ) {
+			for (int _i = 0; _i < 16; _i++)
+				((BYTE*)buf1ptr)[_i] = ((BYTE*)buf1ptr)[_i] ^ ((const BYTE*)buf2ptr)[_i];
 			buf1ptr += 4;
 			buf2ptr += 4;
 		} else {
@@ -2199,23 +2190,10 @@ void RadiusClient::EncryptPasswords(
 #else
 			buf2ptr = (const DWORD*)&digest;
 #endif
-			if( ((unsigned)buf2ptr & 3) || ((unsigned)buf1ptr & 3) ) {
-				((BYTE*)buf1ptr)[0] = ((BYTE*)buf1ptr)[0] ^ ((const BYTE*)buf2ptr)[0];
-				((BYTE*)buf1ptr)[1] = ((BYTE*)buf1ptr)[1] ^ ((const BYTE*)buf2ptr)[1];
-				((BYTE*)buf1ptr)[2] = ((BYTE*)buf1ptr)[2] ^ ((const BYTE*)buf2ptr)[2];
-				((BYTE*)buf1ptr)[3] = ((BYTE*)buf1ptr)[3] ^ ((const BYTE*)buf2ptr)[3];
-				((BYTE*)buf1ptr)[4] = ((BYTE*)buf1ptr)[4] ^ ((const BYTE*)buf2ptr)[4];
-				((BYTE*)buf1ptr)[5] = ((BYTE*)buf1ptr)[5] ^ ((const BYTE*)buf2ptr)[5];
-				((BYTE*)buf1ptr)[6] = ((BYTE*)buf1ptr)[6] ^ ((const BYTE*)buf2ptr)[6];
-				((BYTE*)buf1ptr)[7] = ((BYTE*)buf1ptr)[7] ^ ((const BYTE*)buf2ptr)[7];
-				((BYTE*)buf1ptr)[8] = ((BYTE*)buf1ptr)[8] ^ ((const BYTE*)buf2ptr)[8];
-				((BYTE*)buf1ptr)[9] = ((BYTE*)buf1ptr)[9] ^ ((const BYTE*)buf2ptr)[9];
-				((BYTE*)buf1ptr)[10] = ((BYTE*)buf1ptr)[10] ^ ((const BYTE*)buf2ptr)[10];
-				((BYTE*)buf1ptr)[11] = ((BYTE*)buf1ptr)[11] ^ ((const BYTE*)buf2ptr)[11];
-				((BYTE*)buf1ptr)[12] = ((BYTE*)buf1ptr)[12] ^ ((const BYTE*)buf2ptr)[12];
-				((BYTE*)buf1ptr)[13] = ((BYTE*)buf1ptr)[13] ^ ((const BYTE*)buf2ptr)[13];
-				((BYTE*)buf1ptr)[14] = ((BYTE*)buf1ptr)[14] ^ ((const BYTE*)buf2ptr)[14];
-				((BYTE*)buf1ptr)[15] = ((BYTE*)buf1ptr)[15] ^ ((const BYTE*)buf2ptr)[15];
+			if( (reinterpret_cast<unsigned long>(buf2ptr) & 3) 
+				|| (reinterpret_cast<unsigned long>(buf1ptr) & 3) ) {
+				for (int _i = 0; _i < 16; _i++)
+					((BYTE*)buf1ptr)[_i] = ((BYTE*)buf1ptr)[_i] ^ ((const BYTE*)buf2ptr)[_i];
 				buf1ptr += 4;
 				buf2ptr += 4;
 			} else {
@@ -2249,7 +2227,7 @@ BOOL RadiusClient::GetSocket( RadiusSocket*& socket, unsigned char& id )
 				continue;
 			else {
 				s = activeSockets.GetAt(i);
-				id = _id;
+				id = (unsigned char)_id;
 				break;
 			}
 		} else
@@ -2296,9 +2274,9 @@ BOOL RadiusClient::GetSocket( RadiusSocket*& socket, unsigned char& id )
 			s = NULL;
 
 			if( localAddress == INADDR_ANY )
-				s = CreateSocket( portBase + portIndex );
+				s = CreateSocket((WORD)(portBase + portIndex));
 			else
-				s = CreateSocket( localAddress, portBase + portIndex );
+				s = CreateSocket(localAddress, (WORD)(portBase + portIndex));
 		} while( ((s == NULL) || (!s->IsOpen())) && (--randCount) );
 
 	if( (s == NULL) || (!s->IsOpen()) )
@@ -2339,7 +2317,7 @@ BOOL RadiusClient::GetSocket( RadiusSocket*& socket, unsigned char& id )
 		return FALSE;
 		
 	socket = s;
-	id = i;
+	id = (unsigned char)i;
 
 	return TRUE;
 }
