@@ -14,10 +14,8 @@
 //
 //////////////////////////////////////////////////////////////////
 
-#ifndef __rwlock_h__
-#define __rwlock_h__
-
-#include <ptlib.h>
+#ifndef RWLOCK_H
+#define RWLOCK_H_ "#(@) $Id: rwlock.h,v 1.1.2.3 2004/05/12 17:46:41 zvision Exp $"
 
 // Utility for PReadWriteMutex usage
 
@@ -51,5 +49,5 @@ class WriteUnlock {
 
 extern PReadWriteMutex ConfigReloadMutex;
 
-#endif // __rwlock_h__
+#endif // RWLOCK_H
 
